import { CodegenResult } from '@vue/compiler-core';
import { CompilerError } from '@vue/compiler-core';
import { CompilerOptions } from '@vue/compiler-core';
import { generateCodeFrame } from '@vue/compiler-core';
import { LazyResult } from 'postcss';
import { ParserOptions } from '@vue/compiler-core';
import { RawSourceMap } from 'source-map';
import { Result } from 'postcss';
import { RootNode } from '@vue/compiler-core';
import { SourceLocation } from '@vue/compiler-core';

declare interface AssetURLOptions {
    [name: string]: string[];
}
export { CompilerError }
export { CompilerOptions }

export declare function compileStyle(options: SFCStyleCompileOptions): SFCStyleCompileResults;

export declare function compileStyleAsync(options: SFCStyleCompileOptions): Promise<SFCStyleCompileResults>;

export declare function compileTemplate(options: SFCTemplateCompileOptions): SFCTemplateCompileResults;
export { generateCodeFrame }

export declare function parse(source: string, { sourceMap, filename, sourceRoot, pad, compiler }?: SFCParseOptions): SFCParseResult;

declare type PreprocessLang = 'less' | 'sass' | 'scss' | 'styl' | 'stylus';

export declare interface SFCBlock {
    type: string;
    content: string;
    attrs: Record<string, string | true>;
    loc: SourceLocation;
    map?: RawSourceMap;
    lang?: string;
    src?: string;
}

export declare interface SFCDescriptor {
    filename: string;
    template: SFCTemplateBlock | null;
    script: SFCScriptBlock | null;
    styles: SFCStyleBlock[];
    customBlocks: SFCBlock[];
}

export declare interface SFCParseOptions {
    filename?: string;
    sourceMap?: boolean;
    sourceRoot?: string;
    pad?: boolean | 'line' | 'space';
    compiler?: TemplateCompiler;
}

declare interface SFCParseResult {
    descriptor: SFCDescriptor;
    errors: CompilerError[];
}

export declare interface SFCScriptBlock extends SFCBlock {
    type: 'script';
}

export declare interface SFCStyleBlock extends SFCBlock {
    type: 'style';
    scoped?: boolean;
    module?: string | boolean;
}

export declare interface SFCStyleCompileOptions {
    source: string;
    filename: string;
    id: string;
    map?: RawSourceMap;
    scoped?: boolean;
    trim?: boolean;
    preprocessLang?: PreprocessLang;
    preprocessOptions?: any;
    postcssOptions?: any;
    postcssPlugins?: any[];
}

export declare interface SFCStyleCompileResults {
    code: string;
    map: RawSourceMap | undefined;
    rawResult: LazyResult | Result | undefined;
    errors: Error[];
}

export declare interface SFCTemplateBlock extends SFCBlock {
    type: 'template';
    functional?: boolean;
}

export declare interface SFCTemplateCompileOptions {
    source: string;
    filename: string;
    inMap?: RawSourceMap;
    compiler?: TemplateCompiler;
    compilerOptions?: CompilerOptions;
    preprocessLang?: string;
    preprocessOptions?: any;
    transformAssetUrls?: AssetURLOptions | boolean;
}

export declare interface SFCTemplateCompileResults {
    code: string;
    source: string;
    tips: string[];
    errors: (string | CompilerError)[];
    map?: RawSourceMap;
}

export declare interface TemplateCompiler {
    compile(template: string, options: CompilerOptions): CodegenResult;
    parse(template: string, options: ParserOptions): RootNode;
}

export { }
